package main;

import javafx.application.Application;
import javafx.application.Platform;
import javafx.scene.Scene;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;
import model.ImageAnalysis;
import model.ImageAreaScanResult;
import model.Model;
import model.errorhandling.Error;
import model.errorhandling.ErrorListener;
import model.errorhandling.ErrorMessages;

import java.io.File;
import java.util.Arrays;

public class Main extends Application {
    private static String[] args;
    @Override
    public void start(Stage primaryStage) throws Exception{
        startup(primaryStage);
    }

    private void startup(Stage primaryStage){
        boolean runGui=true;
        boolean mooreNeighborhood=false;
        boolean scanArea=false;
        String filePath=null,outBitsFile=null,outAreaFile=null;
        int[] colorParams=null;
        for(int i=0;i<args.length;i++){
            switch(args[i]){
                case "man":
                case "help":
                case "-man":
                case "-help":
                    printHelp();
                    break;
                case "-nogui":
                    runGui=false;
                    break;
                case "-path":
                    if(args.length<=i+1)
                        ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Wrong parameter passed!\n-path requires a following path-argument!"));
                    filePath=args[i+1];
                    break;
                case "-scanColor":
                    if(args.length<=i+1)
                        ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Wrong parameter passed!\n-scanColor requires a following path-argument!"));
                    String[] data=args[i+1].split(",");
                    try{
                        colorParams=Arrays.stream(data).mapToInt(Integer::parseInt).toArray();
                    }catch (NumberFormatException e){
                        ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Wrong parameter passed!\n-scanColor requires following NUMBERS seperated by \",\"!"));
                    }
                    if(!(colorParams.length==3||colorParams.length==4||colorParams.length==6||colorParams.length==8)) {
                        colorParams = null;
                        ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Wrong parameter passed!\n-scanColor requires 3,4,6 or 8 NUMBERS seperated by \",\"!"));
                    }
                    boolean corrected=false;
                    for(int j=0;j<colorParams.length;j++){
                        if(colorParams[j]<0) {
                            colorParams[j] = 0;
                            corrected=true;
                        }
                        if(colorParams[j]>255) {
                            colorParams[j] = 255;
                            corrected=true;
                        }
                    }
                    if(corrected)
                        ErrorMessages.throwError(new Error(Error.TYPE_WARNING,"The Colorparameter of -scanColor were not in the range of 0 to 255 and were changed to match this min and max value!"));
                    break;
                case "-scanNeighborhood":
                    if(args.length<=i+1)
                        ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Wrong parameter passed!\n-scanNeighborhood requires a following path-argument!"));
                    if(args[i+1].toLowerCase().equals("moore"))
                        mooreNeighborhood=true;
                    else if(args[i+1].toLowerCase().equals("neumann"))
                        mooreNeighborhood=false;
                    else
                        ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Wrong parameter passed!\n-scanNeighborhood requires \"moore\" or \"neumann\" as second parameter!"));
                    break;
                case "-scanArea":
                    if(args.length<=i+1)
                        ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Wrong parameter passed!\n-scanArea requires a following path-argument!"));
                    if(args[i+1].toLowerCase().equals("true"))
                        scanArea=true;
                    else if(args[i+1].toLowerCase().equals("false"))
                        scanArea=false;
                    else
                        ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Wrong parameter passed!\n-scanArea requires \"true\" or \"false\" as second parameter!"));
                    break;
                case "-outBits":
                    if(args.length<=i+1)
                        ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Wrong parameter passed!\n-outBits requires a following path-argument!"));
                    outBitsFile=args[i+1];
                    break;
                case "-outArea":
                    if(args.length<=i+1)
                        ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Wrong parameter passed!\n-outArea requires a following path-argument!"));
                    outAreaFile=args[i+1];
                    break;
            }
        }
        ErrorMessages.addListener(error -> {
            if(error.getType().equals(Error.TYPE_ERROR)) {
                System.err.println("ERROR: " + error.getMessage());
                error.getException().printStackTrace();
            }else{
                System.out.println(error.getType()+": "+error.getMessage());
            }
        });
        Model model=new Model();
        if(runGui)
            initStage(primaryStage);
        if(filePath!=null) {
            ImageAnalysis analysis=model.loadImage(new File(filePath));
            if(colorParams!=null){
                if(!scanArea){
                    boolean[][] result=null;
                    if(colorParams.length==3)
                        result=analysis.scanImage(colorParams[0],colorParams[1],colorParams[2]);
                    else if(colorParams.length==4)
                        result=analysis.scanImage(colorParams[0],colorParams[1],colorParams[2],colorParams[3]);
                    else if(colorParams.length==6)
                        result=analysis.scanImage(colorParams[0],colorParams[1],colorParams[2],colorParams[3],colorParams[4],colorParams[5]);
                    else if(colorParams.length==8)
                        result=analysis.scanImage(colorParams[0],colorParams[1],colorParams[2],colorParams[3],colorParams[4],colorParams[5],colorParams[6],colorParams[7]);
                    if(result!=null&&outBitsFile!=null)
                        model.writeBitsToFile(result,new File(outBitsFile));
                }else{
                    ImageAreaScanResult result=null;
                    if(colorParams.length==3)
                        result=analysis.scanImageArea(colorParams[0],colorParams[1],colorParams[2],mooreNeighborhood);
                    else if(colorParams.length==4)
                        result=analysis.scanImageArea(colorParams[0],colorParams[1],colorParams[2],colorParams[3],mooreNeighborhood);
                    else if(colorParams.length==6)
                        result=analysis.scanImageArea(colorParams[0],colorParams[1],colorParams[2],colorParams[3],colorParams[4],colorParams[5],mooreNeighborhood);
                    else if(colorParams.length==8)
                        result=analysis.scanImageArea(colorParams[0],colorParams[1],colorParams[2],colorParams[3],colorParams[4],colorParams[5],colorParams[6],colorParams[7],mooreNeighborhood);
                    if(result!=null&&outBitsFile!=null)
                        model.writeBitsToFile(result.data,new File(outBitsFile));
                    if(result!=null&&outAreaFile!=null)
                        model.writeAreasToFile(result.areas,new File(outAreaFile));
                }
            }
        }
        if(!runGui)
            Platform.exit();
    }

    private void initStage(Stage primaryStage){
        primaryStage.setTitle("ImageAnalyser");
        primaryStage.setScene(new Scene(new VBox(), 300, 275));
        primaryStage.show();
    }

    public static void main(String[] args) {
        Main.args=args;
        launch(args);
    }
    private static void printHelp(){
        System.out.println("Manpage/Help of ImageAnalyser:");
        System.out.println("\t -man");
        System.out.println("\t -help");
        System.out.println("\t\t Displays this manpage/helpfile");
        System.out.println("\t -nogui silent mode (starts the programm without gui)");
        System.out.println("\t -path <path>");
        System.out.println("\t\t REQUIRED to start an analysis!");
        System.out.println("\t\t Specifies the path of the image to open at startup");
        System.out.println("\t -scanColor <r,g,b>|<a,r,g,b>|<rFrom,rTo,gFrom,gTo,bFrom,bTo>|<aFrom,aTo,rFrom,rTo,gFrom,gTo,bFrom,bTo>");
        System.out.println("\t\t REQUIRED to start an analysis!");
        System.out.println("\t\t Specifies the color to filter. This is are colorvalues between 0 and 255, seperated by a \",\"!");
        System.out.println("\t\t Possible is the exact value Red,Green,Blue / Alpha,Red,Green,Blue or the Ranges.");
        System.out.println("\t\t Examples:");
        System.out.println("\t\t\t Scan white: -scancolor 255,255,255");
        System.out.println("\t\t\t Scan white with half the opacity: -scancolor 127,255,255,255");
        System.out.println("\t\t\t Scan black to darkgrey: -scancolor 0,50,0,50,0,50");
        System.out.println("\t\t\t Scan black to darkgrey with opacity 127 to 255: -scancolor 127,255,0,50,0,50,0,50");
        System.out.println("\t -scanNeighborhood <moore>|<neumann>");
        System.out.println("\t\t Default: neumann");
        System.out.println("\t\t Specifies the neighborhood to search if scanArea is true.");
        System.out.println("\t\t Moore are all 8 neighbors, Neumann just 4 (top,left,bottom,right)");
        System.out.println("\t -scanArea <true>|<false>");
        System.out.println("\t\t Specifies if the search should scan an area or just single bits.");
        System.out.println("\t -outBits <path>");
        System.out.println("\t\t Specifies the path of the bitoutput.");
        System.out.println("\t\t This file lists 0 if the Bit is in the given range, 1 otherwise.");
        System.out.println("\t\t There will be one line of 0 or 1 for every Pixelline.");
        System.out.println("\t -outArea <path>");
        System.out.println("\t\t Specifies the path of the arealist output");
        System.out.println("\t\t This file lists all areas. Each area is written in one line.");
        System.out.println("\t\t An area contains all bits of the area separated by spaces. The coordinates are seperated with a \",\".");
    }
}
