package model;

import javafx.scene.image.Image;
import javafx.scene.image.PixelReader;

import java.io.File;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Patrick on 23.06.2018.
 */
public class ImageAnalysis {
    int[][] image;
    public ImageAnalysis(File imageFile) throws MalformedURLException,IllegalArgumentException {
        this(new Image(imageFile.toURI().toURL().toExternalForm()));
    }
    public ImageAnalysis(Image image){
        this((int)image.getWidth(),(int)image.getHeight());
        PixelReader reader=image.getPixelReader();
        for(int x=0;x<image.getWidth();x++)
            for(int y=0;y<image.getHeight();y++)
                this.image[x][y]=reader.getArgb(x,y);
    }
    private ImageAnalysis(int width,int height){
        this(new int[width][height]);
    }
    public ImageAnalysis(int[][] image){
        if(image.length==0||image[0].length==0)
            throw new IllegalArgumentException("Image size must be greater then 0px!");
        this.image=image;
    }
    public boolean[][] scanImage(int r,int g, int b){
        return scanImage(-1,r,g,b);
    }
    public boolean[][] scanImage(int a,int r,int g, int b){
        return scanImage(a,a,r,r,g,g,b,b);
    }
    public boolean[][] scanImage(int rFrom,int rTo,int gFrom,int gTo,int bFrom,int bTo){
        return scanImage(-1,-1,rFrom,rTo,gFrom,gTo,bFrom,bTo);
    }
    public boolean[][] scanImage(int aFrom,int aTo,int rFrom,int rTo,int gFrom,int gTo,int bFrom,int bTo){
        boolean[][] result=new boolean[image.length][image[0].length];
        for(int i=0;i<image.length;i++)
            for(int j=0;j<image[0].length;j++){
                int pixel=this.image[i][j];
                if(((aFrom==-1&&aTo==-1)||((pixel&0xFF000000)>=aFrom&&(pixel&0xFF000000)<=aTo))&&
                        (((pixel&0x00FF0000)>>16)>=rFrom&&((pixel&0x00FF0000)>>16)<=rTo)&&
                        (((pixel&0x0000FF00)>>8)>=gFrom&&((pixel&0x0000FF00)>>8)<=gTo)&&
                        ((pixel&0x000000FF)>=bFrom&&(pixel&0x000000FF)<=bTo)
                    )
                    result[i][j]=true;
            }
        return result;
    }
    public ImageAreaScanResult scanImageArea(int r, int g, int b, boolean mooreNeighborhood){
        return scanImageArea(r,r,g,g,b,b,mooreNeighborhood);
    }
    public ImageAreaScanResult scanImageArea(int a,int r, int g, int b, boolean mooreNeighborhood){
        return scanImageArea(a,a,r,r,g,g,b,b,mooreNeighborhood);
    }
    public ImageAreaScanResult scanImageArea(int rFrom, int rTo, int gFrom, int gTo, int bFrom, int bTo, boolean mooreNeighborhood){
        return scanImageArea(-1,-1,rFrom,rTo,gFrom,gTo,bFrom,bTo,mooreNeighborhood);
    }
    //default is neumann (4 around), moore checks all 8 neighbors
    public ImageAreaScanResult scanImageArea(int aFrom, int aTo, int rFrom, int rTo, int gFrom, int gTo, int bFrom, int bTo, boolean mooreNeighborhood){
        boolean[][] result=scanImage(aFrom,aTo,rFrom,rTo,gFrom,gTo,bFrom,bTo);
        boolean[][] done=new boolean[result.length][result[0].length];
        List<List<int[]>> data=new ArrayList<>();
        for(int i=0;i<result.length;i++){
            for(int j=0;j<result[0].length;j++){
                if(result[i][j]&&!done[i][j]){
                    ArrayList<int[]> actList=new ArrayList<>();
                    data.add(actList);
                    checkNeighborhood(actList,result,done,i,j,mooreNeighborhood);
                }
            }
        }
        return new ImageAreaScanResult(data,result);
    }
    private void checkNeighborhood(ArrayList<int[]> actList,boolean[][] data,boolean[][] done,int x,int y,boolean mooreNeighborhood){
        if(done[x][y]||!data[x][y])
            return;
        actList.add(new int[]{x,y});
        done[x][y]=true;
        if(x-1>=0)
            checkNeighborhood(actList,data,done,x-1,y,mooreNeighborhood);
        if(y-1>=0)
            checkNeighborhood(actList,data,done,x,y-1,mooreNeighborhood);
        if(x+1<data.length)
            checkNeighborhood(actList,data,done,x+1,y,mooreNeighborhood);
        if(y+1<data[0].length)
            checkNeighborhood(actList,data,done,x,y+1,mooreNeighborhood);
        if (mooreNeighborhood){
            if(x-1>=0&&y-1>=0)
                checkNeighborhood(actList,data,done,x-1,y-1,true);
            if(y-1>=0&&x+1<data.length)
                checkNeighborhood(actList,data,done,x+1,y-1,true);
            if(x-1>=0&&y+1<data[0].length)
                checkNeighborhood(actList,data,done,x-1,y+1, true);
            if(y+1<data[0].length&&x+1<data.length)
                checkNeighborhood(actList,data,done,x+1,y+1, true);
        }
    }
}
