package model;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import model.errorhandling.Error;
import model.errorhandling.ErrorMessages;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.nio.file.Files;
import java.nio.file.OpenOption;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import java.util.function.IntFunction;
import java.util.function.IntUnaryOperator;
import java.util.stream.Collectors;

/**
 * Created by Patrick on 23.06.2018.
 */
public class Model {
    private ObservableList<ImageAnalysis> analysisList;
    public Model(){
        analysisList=FXCollections.observableArrayList();
    }
    public ImageAnalysis loadImage(File imageFile){
        try {
            ImageAnalysis analysis=new ImageAnalysis(imageFile);
            analysisList.add(analysis);
            return analysis;
        } catch (MalformedURLException e) {
            ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"A Problem with the Imagepath occured.\n"+
                    "Please check the Path: \""+imageFile+"\"",e));
        }catch (IllegalArgumentException e){
            ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"The given File is no Image or the Imageformat is not supported.\n"+
                    "Please check the Path: \""+imageFile+"\"",e));
        }
        return null;
    }
    public void writeBitsToFile(boolean[][] data,File file){
        StringBuilder builder=new StringBuilder();
        for(int i=0;i<data[0].length;i++) {
            for (int j = 0; j < data.length; j++)
                builder.append(data[j][i]?"1":"0");
            builder.append("\r\n");
        }
        try {
            Files.write(Paths.get(file.toURI()),builder.toString().getBytes());
        } catch (IOException e) {
            ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Error saving bit-file: \""+file+"\"!",e));
        }
    }

    public void writeAreasToFile(List<List<int[]>> areas, File file) {
        String result=areas.stream().map(ints -> ints.stream().map(ints1 -> Arrays.stream(ints1).mapToObj(value -> ""+value).collect(Collectors.joining(","))).collect(Collectors.joining(" "))).collect(Collectors.joining("\r\n"));
        try {
            Files.write(Paths.get(file.toURI()),result.getBytes());
        } catch (IOException e) {
            ErrorMessages.throwError(new Error(Error.TYPE_ERROR,"Error saving area-file: \""+file+"\"!",e));
        }
    }
}
